"""Quote adapters for the Deal Desk seller quote loop."""

from .base import QuoteAdapter, QuoteAdapterSupportResult, QuoteAdapterSupportStatus
from .router import resolve_quote_adapter, resolve_quote_adapter_for_environment, seller_quote_support
from .salesforce_cpq import SalesforceCpqQuoteAdapter
from .salesforce_standard import SalesforceStandardQuoteAdapter

__all__ = [
    "QuoteAdapter",
    "QuoteAdapterSupportResult",
    "QuoteAdapterSupportStatus",
    "SalesforceCpqQuoteAdapter",
    "SalesforceStandardQuoteAdapter",
    "resolve_quote_adapter",
    "resolve_quote_adapter_for_environment",
    "seller_quote_support",
]
