from fmatch.saas.operators.split_fullname import split_fullname


def test_names():
    assert split_fullname("SMITH, JOHN")["value"] == ["John", "Smith"]
    assert split_fullname("John A. Smith Jr.")["value"] == ["John", "Smith"]

