from src.fmatch.saas.operators.normalize_country import normalize_country


def test_country():
    assert normalize_country("USA")["value"] == "US"
    assert normalize_country("UAE")["value"] == "AE"
    assert normalize_country("Czech Republic")["value"] == "CZ"

