"""Shared FullEnrich helpers for CLI and SaaS integrations."""

from .constants import (
    API_BASE_V1,
    API_BASE_V2,
    CHECK_PARAMS,
    COMPANY_ENRICH_PATH,
    CONTACT_BULK_PATH,
    CREDITS_PATH,
    DEFAULT_API_VERSION,
    DEFAULT_CONTACT_FIELDS,
    DEFAULT_CONTACT_OUTPUT_FIELDS,
    FullEnrichAPIVersion,
    UI_FIELD_TO_ENRICH_FIELD,
    VERIFY_PATH,
    api_base,
    build_url,
    map_output_fields_to_enrich_fields,
    parse_credit_balance,
    CONTACT_ERROR_STATUSES,
    CONTACT_NOT_FOUND_STATUSES,
    JOB_PENDING_STATUSES,
    JOB_TERMINAL_STATUSES,
    JOB_FAILURE_STATUSES,
    normalize_job_status,
    is_terminal_status,
)
from .schema import (
    ContactEnrichmentInput,
    ContactEnrichmentResult,
    EnrichmentResultStatus,
)
from .validation import (
    derive_contact_name,
    extract_custom_value,
    extract_domain_from_email,
    is_valid_contact_input,
)
from .payload import (
    build_bulk_payload,
    build_bulk_payload_v1,
    build_bulk_payload_v2,
    build_contact_payload,
    build_contact_payload_v1,
    build_contact_payload_v2,
    build_custom_metadata,
    extract_enrichment_id,
    normalize_contact_list,
    parse_bulk_response,
    parse_bulk_response_v1,
    parse_bulk_response_v2,
    parse_contact_response,
    parse_contact_response_v1,
    parse_contact_response_v2,
)

__all__ = [
    "API_BASE_V1",
    "API_BASE_V2",
    "CONTACT_BULK_PATH",
    "COMPANY_ENRICH_PATH",
    "CREDITS_PATH",
    "VERIFY_PATH",
    "DEFAULT_API_VERSION",
    "DEFAULT_CONTACT_FIELDS",
    "DEFAULT_CONTACT_OUTPUT_FIELDS",
    "CHECK_PARAMS",
    "FullEnrichAPIVersion",
    "api_base",
    "build_url",
    "parse_credit_balance",
    "CONTACT_ERROR_STATUSES",
    "CONTACT_NOT_FOUND_STATUSES",
    "JOB_PENDING_STATUSES",
    "JOB_TERMINAL_STATUSES",
    "JOB_FAILURE_STATUSES",
    "normalize_job_status",
    "is_terminal_status",
    "ContactEnrichmentInput",
    "ContactEnrichmentResult",
    "EnrichmentResultStatus",
    "derive_contact_name",
    "extract_custom_value",
    "extract_domain_from_email",
    "is_valid_contact_input",
    "build_bulk_payload",
    "build_bulk_payload_v1",
    "build_bulk_payload_v2",
    "build_contact_payload",
    "build_contact_payload_v1",
    "build_contact_payload_v2",
    "build_custom_metadata",
    "extract_enrichment_id",
    "normalize_contact_list",
    "parse_bulk_response",
    "parse_bulk_response_v1",
    "parse_bulk_response_v2",
    "parse_contact_response",
    "parse_contact_response_v1",
    "parse_contact_response_v2",
]
