from src.fmatch.saas.operators.normalize_state import normalize_state


def test_us_states():
    assert normalize_state("Delaware")["value"] == "Delaware"
    assert normalize_state("de")["value"] == "Delaware"
    assert normalize_state("Ontario", country="CA")["value"] == "Ontario"
    assert normalize_state("California", country="CA")["value"] == ""

