"""GraphQL Metadata API query helpers for Tableau."""

METADATA_DOCTOR_QUERY = "query { __typename }"

DEFAULT_FIELDS_QUERY = """
query DatasourceFields($datasourceId: ID!) {
  datasource(id: $datasourceId) {
    id
    name
    fields {
      name
      dataType
      description
      isHidden
      role
    }
  }
}
"""

DEFAULT_LINEAGE_QUERY = """
query DatasourceLineage($datasourceId: ID!) {
  datasource(id: $datasourceId) {
    id
    name
    upstreamDatasources {
      id
      name
    }
    downstreamWorkbooks {
      id
      name
    }
  }
}
"""
